Package: earth
Version: 5.3.3
Title: Multivariate Adaptive Regression Splines
Author: Stephen Milborrow.  Derived from mda:mars by Trevor Hastie and
    Rob Tibshirani.  Uses Alan Miller's Fortran utilities
    with Thomas Lumley's leaps wrapper.
Maintainer: Stephen Milborrow <milbo@sonic.net>
Depends: R (>= 3.4.0), Formula (>= 1.2-3), plotmo (>= 3.6.0),
Suggests: gam (>= 1.20), mgcv (>= 1.8-26), mda (>= 0.5-2), MASS (>=
        7.3-51)
Description: Build regression models using the techniques in Friedman's
    papers "Fast MARS" and "Multivariate Adaptive Regression
    Splines" <doi:10.1214/aos/1176347963>.
    (The term "MARS" is trademarked and thus not used in
    the name of the package.)
License: GPL-3
LazyData: yes
URL: http://www.milbo.users.sonic.net/earth/
Packaged: 2024-02-26 12:13:39 UTC; milbo
Repository: CRAN
Date/Publication: 2024-02-26 14:00:11 UTC
RoxygenNote: 5.1.2
NeedsCompilation: yes
Built: R 4.2.3; aarch64-apple-darwin20; 2024-02-26 15:23:43 UTC; unix
Archs: earth.so.dSYM
