useDynLib("earth")

import(stats)
importFrom(plotmo, plotmo.singles, plotmo.pairs, plotmo.y)
importFrom(Formula, Formula, model.part)

export(contr.earth.response)
export(earth)
export(earth_plotmodsel)
export(evimp)
export(expand.bpairs)
export(mars.to.earth)
export(plot.earth.models)
export(plotd)

S3method(plotmo.singles, earth)
S3method(plotmo.pairs, earth)
S3method(plotmo.y, earth)
S3method(plotmo.pairs, bagEarth)
S3method(plotmo.y, bagEarth)

S3method(anova, earth)
S3method(case.names, earth)
S3method(coef, earth)
S3method(coef, varmod)
S3method(deviance, earth)
S3method(earth, default)
S3method(earth, formula)
S3method(earth, fit)
S3method(effects, earth)
S3method(extractAIC, earth)
S3method(expand.bpairs, default)
S3method(expand.bpairs, formula)
S3method(family, earth)
S3method(fitted, earth)
S3method(fitted.values, earth)
S3method(format, earth)
S3method(format, lm)
S3method(hatvalues, earth)
S3method(model.matrix, earth)
S3method(plot, earth)
S3method(plot, earth.models)
S3method(plot, evimp)
S3method(plot, varmod)
S3method(predict, earth)
S3method(predict, varmod)
S3method(print, earth)
S3method(print, evimp)
S3method(print, summary.earth)
S3method(print, summary.varmod)
S3method(print, varmod)
S3method(resid, earth)
S3method(residuals, earth)
S3method(summary, earth)
S3method(summary, varmod)
S3method(update, earth)
S3method(variable.names, earth)
S3method(weights, earth)

importFrom("grDevices", "as.graphicsAnnot", col2rgb, "heat.colors",
           "xy.coords")

importFrom("graphics", "abline", "axis", "legend", "lines", "mtext",
           "par", "plot", "points", "polygon", "rect", "segments",
           "strheight", "strwidth", "text", "xinch", "yinch")

importFrom("utils", "assignInMyNamespace", "flush.console", "head",
           "memory.size", "str")
